@@ -411,6 +411,9 @@ namespace usbguard
     catch (const RuleParserError& ex) {
       throw Exception("Rules", _nss.getSourceInfo(), ex.hint());
     }
+    catch (const Exception& ex) {
+      throw ex;
+    }
     catch (const std::exception& ex) {
       throw Exception("Rules", _nss.getSourceInfo(), ex.what());
     }
@@ -443,12 +446,25 @@ namespace usbguard
   void Daemon::parseIPCAccessControlFilename(const std::string& basename, std::string* const ptr_user,
     std::string* const ptr_group)
   {
+    // There are five supported forms:
+    // - "<user>:<group>"
+    // - "<user>:"
+    // - "<user>"
+    // - ":<group>"
+    // - ":"
     const auto ug_separator = basename.find_first_of(":");
     const bool has_group = ug_separator != std::string::npos;
     const std::string user = basename.substr(0, ug_separator);
     const std::string group = has_group ? basename.substr(ug_separator + 1) : std::string();
-    checkIPCAccessControlName(user);
-    checkIPCAccessControlName(group);
+
+    if (! user.empty()) {
+      checkIPCAccessControlName(user);
+    }
+
+    if (! group.empty()) {
+      checkIPCAccessControlName(group);
+    }
+
     *ptr_user = user;
     *ptr_group = group;
   }
@@ -849,6 +865,9 @@ namespace usbguard
       _dm->applyDevicePolicy(device->getID(),
         matched_rule->getTarget());
     const bool target_changed = target_old != device_post->getTarget();
+    std::shared_ptr<const Rule> device_rule = \
+      device_post->getDeviceRule(/*with_port=*/true,
+        /*with_parent_hash=*/true);
 
     if (target_changed || matched_rule->getRuleID() == Rule::ImplicitID) {
       if (target_changed) {
@@ -860,9 +879,6 @@ namespace usbguard
         USBGUARD_LOG(Debug) << "Implicit rule matched";
       }
 
-      std::shared_ptr<const Rule> device_rule = \
-        device_post->getDeviceRule(/*with_port=*/true,
-          /*with_parent_hash=*/true);
       DevicePolicyChanged(device->getID(),
         target_old,
         device_post->getTarget(),
@@ -870,6 +886,10 @@ namespace usbguard
         matched_rule->getRuleID());
     }
 
+    DevicePolicyApplied(device->getID(),
+      device_post->getTarget(),
+      device_rule->toString(),
+      matched_rule->getRuleID());
     matched_rule->updateMetaDataCounters(/*applied=*/true);
     audit_event.success();
   }
